@@ -5,7 +5,7 @@
  * object level logging, and fully-qualified object names for all DML and DDL
  * statements where possible (See README.md for details).
  *
- * Copyright (c) 2014-2022, PostgreSQL Global Development Group
+ * Copyright (c) 2014-2019, PostgreSQL Global Development Group
  *------------------------------------------------------------------------------
  */
 #include "postgres.h"
@@ -13,12 +13,12 @@
 #include "access/htup_details.h"
 #include "access/sysattr.h"
 #include "access/xact.h"
+#include "access/relation.h"
 #include "catalog/catalog.h"
 #include "catalog/objectaccess.h"
 #include "catalog/pg_class.h"
 #include "catalog/namespace.h"
 #include "commands/dbcommands.h"
-#include "commands/extension.h"
 #include "catalog/pg_proc.h"
 #include "commands/event_trigger.h"
 #include "executor/executor.h"
@@ -63,6 +63,7 @@ PG_FUNCTION_INFO_V1(pgaudit_sql_drop);
 #define LOG_READ        (1 << 3)    /* SELECTs */
 #define LOG_ROLE        (1 << 4)    /* GRANT/REVOKE, CREATE/ALTER/DROP ROLE */
 #define LOG_WRITE       (1 << 5)    /* INSERT, UPDATE, DELETE, TRUNCATE */
+#define LOG_MISC_SET    (1 << 6)    /* SET ... */
 
 #define LOG_NONE        0               /* nothing */
 #define LOG_ALL         (0xFFFFFFFF)    /* All */
@@ -80,6 +81,7 @@ static int auditLogBitmap = LOG_NONE;
 #define CLASS_DDL       "DDL"
 #define CLASS_FUNCTION  "FUNCTION"
 #define CLASS_MISC      "MISC"
+#define CLASS_MISC_SET  "MISC_SET"
 #define CLASS_READ      "READ"
 #define CLASS_ROLE      "ROLE"
 #define CLASS_WRITE     "WRITE"
@@ -481,19 +483,6 @@ log_audit_event(AuditEventStackItem *stackItem)
     MemoryContext contextOld;
     StringInfoData auditStr;
 
-    /*
-     * Skip logging script statements if an extension is currently being created
-     * or altered. PostgreSQL reports the statement text for each statement in
-     * the script as the entire script text, which can blow up the logs. The
-     * create/alter statement will still be logged.
-     *
-     * Since a superuser is responsible for determining which extensions are
-     * available, and in most cases installing them, it should not be necessary
-     * to log each statement in the script.
-     */
-    if (creating_extension)
-        return;
-
     /* If this event has already been logged don't log it again */
     if (stackItem->auditEvent.logged)
         return;
@@ -626,6 +615,15 @@ log_audit_event(AuditEventStackItem *stackItem)
                     class = LOG_FUNCTION;
                     break;
 
+                /*
+                 * SET statements reported as MISC but filtered by MISC_SET
+                 * flags to maintain existing functionality.
+                 */
+                case T_VariableSetStmt:
+                    className = CLASS_MISC;
+                    class = LOG_MISC_SET;
+                    break;
+
                 default:
                     break;
             }
@@ -984,7 +982,7 @@ log_select_dml(Oid auditOid, List *rangeTabls)
     foreach(lr, rangeTabls)
     {
         Oid relOid;
-        Oid relNamespaceOid;
+        Relation rel;
         RangeTblEntry *rte = lfirst(lr);
 
         /* We only care about tables, and can ignore subqueries etc. */
@@ -1007,10 +1005,13 @@ log_select_dml(Oid auditOid, List *rangeTabls)
          * false) then filter out any system relations here.
          */
         relOid = rte->relid;
-        relNamespaceOid = get_rel_namespace(relOid);
+        rel = relation_open(relOid, NoLock);
 
-        if (!auditLogCatalog && IsSystemNamespace(relNamespaceOid))
+        if (!auditLogCatalog && IsCatalogNamespace(RelationGetNamespace(rel)))
+        {
+            relation_close(rel, NoLock);
             continue;
+        }
 
         /*
          * Default is that this was not through a grant, to support session
@@ -1069,12 +1070,10 @@ log_select_dml(Oid auditOid, List *rangeTabls)
         switch (rte->relkind)
         {
             case RELKIND_RELATION:
-            case RELKIND_PARTITIONED_TABLE:
                 auditEventStack->auditEvent.objectType = OBJECT_TYPE_TABLE;
                 break;
 
             case RELKIND_INDEX:
-            case RELKIND_PARTITIONED_INDEX:
                 auditEventStack->auditEvent.objectType = OBJECT_TYPE_INDEX;
                 break;
 
@@ -1109,8 +1108,10 @@ log_select_dml(Oid auditOid, List *rangeTabls)
 
         /* Get a copy of the relation name and assign it to object name */
         auditEventStack->auditEvent.objectName =
-            quote_qualified_identifier(
-                get_namespace_name(relNamespaceOid), get_rel_name(relOid));
+            quote_qualified_identifier(get_namespace_name(
+                                           RelationGetNamespace(rel)),
+                                       RelationGetRelationName(rel));
+        relation_close(rel, NoLock);
 
         /* Perform object auditing only if the audit role is valid */
         if (auditOid != InvalidOid)
@@ -1218,7 +1219,7 @@ log_function_execute(Oid objectId)
      * Logging execution of all pg_catalog functions would make the log
      * unusably noisy.
      */
-    if (IsSystemNamespace(proc->pronamespace))
+    if (IsCatalogNamespace(proc->pronamespace))
     {
         ReleaseSysCache(proctup);
         return;
@@ -1374,27 +1375,8 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
         /* Process top level utility statement */
         if (context == PROCESS_UTILITY_TOPLEVEL)
         {
-            /*
-             * If the stack is not empty then the only allowed entries are open
-             * select, show, and explain cursors
-             */
             if (auditEventStack != NULL)
-            {
-                AuditEventStackItem *nextItem = auditEventStack;
-
-                do
-                {
-                    if (nextItem->auditEvent.commandTag != T_SelectStmt &&
-                        nextItem->auditEvent.commandTag != T_VariableShowStmt &&
-                        nextItem->auditEvent.commandTag != T_ExplainStmt)
-                    {
-                        elog(ERROR, "pgaudit stack is not empty");
-                    }
-
-                    nextItem = nextItem->next;
-                }
-                while (nextItem != NULL);
-            }
+                elog(ERROR, "pgaudit stack is not empty");
 
             stackItem = stack_push();
             stackItem->auditEvent.paramList = copyParamList(params);
@@ -1416,31 +1398,6 @@ pgaudit_ProcessUtility_hook(PlannedStmt *pstmt,
             stackItem->auditEvent.commandTag == T_DoStmt &&
             !IsAbortedTransactionBlockState())
             log_audit_event(stackItem);
-
-        /*
-         * If this is a create/alter extension command log it before calling
-         * the next ProcessUtility hook. Otherwise, any warnings will be emitted
-         * before the create/alter is logged and errors will prevent it from
-         * being logged at all.
-         */
-        if (auditLogBitmap & LOG_DDL &&
-            (stackItem->auditEvent.commandTag == T_CreateExtensionStmt ||
-                stackItem->auditEvent.commandTag == T_AlterExtensionStmt) &&
-            !IsAbortedTransactionBlockState())
-            log_audit_event(stackItem);
-
-        /*
-         * A close will free the open cursor which will also free the close
-         * audit entry. Immediately log the close and set stackItem to NULL so
-         * it won't be logged later.
-         */
-        if (stackItem->auditEvent.commandTag == T_ClosePortalStmt)
-        {
-            if (auditLogBitmap & LOG_MISC && !IsAbortedTransactionBlockState())
-                log_audit_event(stackItem);
-
-            stackItem = NULL;
-        }
     }
 
     /* Call the standard process utility chain. */
@@ -1548,9 +1505,7 @@ pgaudit_ddl_command_end(PG_FUNCTION_ARGS)
         CreateCommandTag(eventData->parsetree);
 
     /* Return objects affected by the (non drop) DDL statement */
-    query = "SELECT pg_catalog.upper(object_type),\n"
-            "       object_identity,\n"
-            "       pg_catalog.upper(command_tag)\n"
+    query = "SELECT UPPER(object_type), object_identity, UPPER(command_tag)\n"
             "  FROM pg_catalog.pg_event_trigger_ddl_commands()";
 
     /* Attempt to connect */
@@ -1649,11 +1604,11 @@ pgaudit_sql_drop(PG_FUNCTION_ARGS)
     contextOld = MemoryContextSwitchTo(contextQuery);
 
     /* Return objects affected by the drop statement */
-    query = "SELECT pg_catalog.upper(object_type),\n"
-            "       object_identity\n"
-            "  FROM pg_catalog.pg_event_trigger_dropped_objects()\n"
-            " WHERE pg_catalog.lower(object_type) operator(pg_catalog.<>) 'type'\n"
-            "   AND schema_name operator(pg_catalog.<>) 'pg_toast'";
+    query = "SELECT UPPER(object_type),\n"
+        "       object_identity\n"
+        "  FROM pg_catalog.pg_event_trigger_dropped_objects()\n"
+        " WHERE lower(object_type) <> 'type'\n"
+        "   AND schema_name <> 'pg_toast'";
 
     /* Attempt to connect */
     result = SPI_connect();
@@ -1755,7 +1710,9 @@ check_pgaudit_log(char **newVal, void **extra, GucSource source)
         else if (pg_strcasecmp(token, CLASS_FUNCTION) == 0)
             class = LOG_FUNCTION;
         else if (pg_strcasecmp(token, CLASS_MISC) == 0)
-            class = LOG_MISC;
+            class = LOG_MISC | LOG_MISC_SET;
+        else if (pg_strcasecmp(token, CLASS_MISC_SET) == 0)
+            class = LOG_MISC_SET;
         else if (pg_strcasecmp(token, CLASS_READ) == 0)
             class = LOG_READ;
         else if (pg_strcasecmp(token, CLASS_ROLE) == 0)
